#include "gis.h"
#include "mask.h"

extern char *malloc();

init_mask_rules (mask)
    Mask *mask;
{
    mask->list = NULL;
}

init_d_mask_rules (d_mask)
    d_Mask *d_mask;
{
    d_mask->list = NULL;
}

add_mask_rule (mask, a, b, inf)
    Mask *mask;
    long a, b;
    int inf;
{
    Interval *I;

    I = (Interval *) G_malloc (sizeof(Interval));
    I->low  = a <= b ? a : b ;
    I->high = a >= b ? a : b ;
    I->inf  = inf;
    I->next = mask->list;
    mask->list = I;
}

add_d_mask_rule (d_mask, a, b, inf)
    d_Mask *d_mask;
    double a, b;
    int inf;
{
    d_Interval *I;

    I = (d_Interval *) G_malloc (sizeof(d_Interval));
    I->low  = a <= b ? a : b ;
    I->high = a >= b ? a : b ;
    I->inf  = inf;
    I->next = d_mask->list;
    d_mask->list = I;
}

mask_cell_array (cell, ncols, mask, invert)
    CELL *cell;
    int ncols;
    Mask *mask;
    int invert;
{
    long x;

    while (ncols-- > 0)
    {
	x = *cell;
	if (mask_select (&x, mask, invert))
	    *cell++ = x;
	else
	    G_set_c_null_value(cell++, 1);
    }
}

mask_d_cell_array (dcell, ncols, mask, invert)
    DCELL *dcell;
    int ncols;
    d_Mask *mask;
    int invert;
{
    DCELL x;

    while (ncols-- > 0)
    {
	x = *dcell;
	if (mask_d_select (&x, mask, invert))
	    *dcell++ = x;
	else
	    G_set_d_null_value(dcell++, 1);
    }
}

mask_select (x, mask, invert)
    long *x;
    Mask *mask;
    int invert;
{
    Interval *I;

    if (mask->list == NULL) return 1;
    for (I = mask->list; I; I = I->next)
    {
	if (mask_match_interval (*x, I))
	{
	    if (invert) return 0;
	    return 1;
	}
    }
    return invert;
}

mask_d_select (x, mask, invert)
    DCELL *x;
    d_Mask *mask;
    int invert;
{
    d_Interval *I;

    if (mask->list == NULL) return 1;
    for (I = mask->list; I; I = I->next)
    {
	if (mask_match_d_interval (*x, I))
	{
	    if (invert) return 0;
	    return 1;
	}
    }
    return invert;
}

mask_match_interval (x, I)
    long x;
    Interval *I;
{
    if (I->inf < 0)
	return x <= I->low;

    if (I->inf > 0)
	return x >= I->high;

    return x >= I->low && x <= I->high;
}

mask_match_d_interval (x, I)
    DCELL x;
    d_Interval *I;
{
    if (I->inf < 0)
	return x <= I->low;

    if (I->inf > 0)
	return x >= I->high;

    return x >= I->low && x <= I->high;
}
