#ifdef DEC_ALPHA
#include <float.h>
#endif
#include "G.h"
/*************************************************************
*   G_set_f_null_value(f, n)
*      FCELL *f;
*      int n;
*
*   G_set_d_null_value(d, n)
*      DCELL *d;
*      int n;
*
*   G_set_c_null_value(c, n)
*      CELL *c;
*      int n;
*
*   G_set_null_value(c, n, type)
*      CELL *c;
*      int n;
*      RASTER_MAP_TYPE type;
*
*   set n cells to null.
*
***************************************************************
*
*   G_is_f_null_value(f)
*      FCELL *f;
*      int n;
*
*   G_is_d_null_value(d)
*      FCELL *d;
*      int n;
*
*   G_is_c_null_value(c)
*      CELL *c;
*      int n;
*
*   G_is_null_value(c, type)
*      FCELL c;
*      int n;
*      RASTER_MAP_TYPE type;
*
*   check if cell is null
*   
***************************************************************
*   
*   G_insert_f/d/c_null_values (cell, flags, count) 
*   
*      [D/F]CELL *cell; 
*      char *flags; 
*      int count; 
*   
*      For each of the count flags which is true(!=0), set the 
*      corresponding cell to the NULL value.
*   
***************************************************************/

#define FCB          G__.fileinfo[fd]
/*--------------------------------------------------------------------------*/

/* convert type "RASTER_MAP_TYPE" into index */
#define F2I(map_type) \
	(map_type == CELL_TYPE ? 0 : (map_type == FCELL_TYPE ? 1 : 2))

static FCELL FCELL_NULL_PATTERN;
static DCELL DCELL_NULL_PATTERN;
/* the float and double patterns are not used right now
to check numbers abainst. They are only used to set a null
value. To check if the value is null comparison to itself is used */

static CELL CELL_NULL_PATTERN;

G__check_null_bit(flags, bit_num, n)
   unsigned char *flags;
   int n, bit_num;
{
   int ind, offset;
   char msg[200];

   /* find the index of the unsigned char in which this bit appears */

   ind = G__null_bitstream_size(bit_num + 1) - 1;

   /* find how many unsigned chars the buffer with bit_num+1 (counting from 0
      has and subtract 1 to get unsigned char index */

   if(ind > G__null_bitstream_size(n) - 1 )
   {
      sprintf(msg,"G__check_null_bit: can't access inex %d. Size of flags is %d (bit # is %d", ind, G__null_bitstream_size(n) - 1, bit_num);
      G_warning(msg);
      return -1;
   }
   offset = (ind+1)*8 - bit_num - 1;

   return ((flags[ind] & ( (unsigned char) 1 << offset)) != 0);
}

G__set_flags_from_01_random(zero_ones, flags, col, n, ncols)
/* given array of 0/1 of length n starting from column col
   set the corresponding  bits of flags; 
   total number of bits in flags is ncols */

   unsigned char *flags;
   char *zero_ones;
   int n, ncols;
{
   int i, k, count, size;
   unsigned char v;

   if(col==0 && n == ncols)
   {
       G__convert_01_flags(zero_ones, flags, n);
       return 0;
   }
   count = 0;
   size = G__null_bitstream_size(ncols);
   for (i = 0; i < size; i++)
   {
      v = 0;
      k = 8;
      while (k-- > 0)
      {
	 if (count >= col && count < (col+ n))  
	    v = v | ((unsigned char) zero_ones[count - col] << k);
         else if(count < ncols)
            v = v | ((unsigned char) G__check_null_bit(flags, count, ncols) 
                                                   <<k);
            /* otherwise  keep this bit the same as it was */
         count++;
      }
      flags[i] = v;
    }
    return 1;

}

G__convert_01_flags(zero_ones, flags, n)
   unsigned char *flags;
   char *zero_ones;
   int n;
{
   int i, k, count, size;
   unsigned char *v;

   /* pad the flags with 0's to make size multiple of 8 */
   v = flags;
   size = G__null_bitstream_size(n);
   count = 0;
   for (i = 0; i < size; i++)
   {
      *v = 0;
      k = 8;
      while (k-- > 0)
      {
	 if (count < n)  
	    *v = *v | ((unsigned char) zero_ones[count] << k);
         count++;
      }
      v++;
    }
}

G__convert_flags_01(zero_ones, flags, n)
   unsigned char *flags;
   char *zero_ones;
   int n;
{
   int i, k, count, size;
   unsigned char *v;

   count = 0;
   v = flags;
   size = G__null_bitstream_size(n);
   for (i = 0; i < size; i++)
   {
      k = 8;
      while (k-- > 0)
      {
	 if (count < n) 
	 {
	     zero_ones[count] = ((*v & ( (unsigned char) 1 << k)) != 0);
             count++;
	 }
      }
      v++;
    }
}

G__init_null_bits(flags, cols)
   int cols;
   unsigned char *flags;
{
   int i, size;
   unsigned char *v;

   /* pad the flags with 0's to make size multiple of 8 */
   v = flags;
   size = G__null_bitstream_size(cols);
   for (i = 0; i < size; i++)
   {
      if((i+1) * 8 <= cols)
         *v = (unsigned char) 255;
      else
         *v = (unsigned char) 255 << ((i+1) * 8 - cols);
      v++;
   }
}

/* since on some architectures (sgi) the float whose all
bits are set to one is changed internally when it is passed
as an argument to some function ( some bits are changed to 0),
we need to store machine dependant bit pattern as  FCELL_NULL_PATTERN
and then make byte-wise comparison to this bit pattern.
*/

int
G__init_null_patterns()
{
    FCELL f;
    DCELL d;
    CELL c;

    G_set_f_null_value(&f, 1);
    set_f_pattern(f);

    G_set_d_null_value(&d, 1);
    set_d_pattern(d);

    G_set_c_null_value(&c, 1);
    set_pattern(c);
}

static int
set_f_pattern(f)
  FCELL f;
{
  FCELL_NULL_PATTERN = f;
}


static int
set_d_pattern(d)
  DCELL d;
{
  DCELL_NULL_PATTERN = d;
}


static int
set_pattern(c)
  CELL c;
{
  CELL_NULL_PATTERN = c;
}

G__set_null_value(rast, n, null_is_zero, data_type)
    void *rast;
    int n, null_is_zero;
    RASTER_MAP_TYPE data_type;
{
    if(null_is_zero)
    {
       G_zero((char *) rast, n * G_raster_size(data_type));
       return;
    }
    G_set_null_value(rast, n, data_type);
} 

G_set_null_value(buf, n, data_type)
    void *buf;
    int n;
    RASTER_MAP_TYPE data_type;
{
    switch (data_type)
    {
       case CELL_TYPE: G_set_c_null_value((CELL *) buf, n); break;
       case FCELL_TYPE: G_set_f_null_value((FCELL *) buf, n); break;
       case DCELL_TYPE: G_set_d_null_value((DCELL *) buf, n); break;
    }
} 

G_set_f_null_value(f, n)
    FCELL *f;
    int n;
{
#ifdef DEC_ALPHA
    *f = FLT_MAX;
#else
    unsigned char *p;

    p = (unsigned char *) f;
    G__init_null_bits(p, n * sizeof(FCELL) * 8);
#endif
} 

G_set_d_null_value(d, n)
    DCELL *d;
    int n;
{
#ifdef DEC_ALPHA
+ /*
+       it appears DBL_MAX can not be assigned to a float on an alpha
+       so use FLT_MAX instead
+       *d = DBL_MAX;
+ */
    *d = FLT_MAX;
#else
    unsigned char *p;

    p = (unsigned char *) d;
    G__init_null_bits(p, n * sizeof(DCELL) * 8);
#endif
} 

G_set_c_null_value(c, n)
    CELL *c;
    int n;
{
    int i;
    unsigned char *p;

    p = (unsigned char *) c;

    G__init_null_bits(p, n * sizeof(CELL) * 8);
    i = n;
    while(i-- > 0)
    {
       *p = *p >> 1;
       p += sizeof(CELL);
    }
} 

G_is_f_null_value(f)
    FCELL *f;
{
#ifdef DEC_ALPHA
    return (*f == FLT_MAX );
#else
/*    if(!(*f<0.) && !(*f>0.) && !(*f==0.))*/
    return (*f!=*f);
#endif
}

G_is_d_null_value(d)
    DCELL *d;
{

#ifdef DEC_ALPHA
    return (*d == FLT_MAX );
#else
/*    if(!(*d<0.) && !(*d>0.) && !(*d==0.))*/
    return (*d!=*d);
#endif
}

G_is_c_null_value(c)
    CELL *c;
{
    int i;
    CELL temp;

    temp = *c;
    for(i = 0; i < sizeof (CELL); i++)
    {
       if(((unsigned char*)&temp)[i] != 
               ((unsigned char*)&CELL_NULL_PATTERN)[i])
              return 0;
    }
    return 1;
}

G_is_null_value(rast, data_type)
    void *rast;
    RASTER_MAP_TYPE data_type;
{
    switch(data_type)
    {
       case CELL_TYPE: return G_is_c_null_value((CELL *) rast); 
       case FCELL_TYPE: return G_is_f_null_value((FCELL *) rast);
       case DCELL_TYPE: return G_is_d_null_value((DCELL *) rast);
       default: G_warning("G_is_null_value: wrong data type!");
		return 0;
    }
}

G_insert_f_null_values(fcell, null_row, ncols)
    FCELL *fcell;
    char *null_row;
    int ncols;
{
    return embed_given_nulls((void *) fcell, null_row, FCELL_TYPE, ncols);
}

G_insert_d_null_values(dcell, null_row, ncols)
    DCELL *dcell;
    char *null_row;
    int ncols;
{
    return embed_given_nulls((void *) dcell, null_row, DCELL_TYPE, ncols);
}

G_insert_c_null_values(cell, null_row, ncols)
    CELL *cell;
    char *null_row;
    int ncols;
{
    return embed_given_nulls((void *) cell, null_row, CELL_TYPE, ncols);
}

G_insert_null_values(rast, null_row, ncols, data_type)
    void *rast;
    char *null_row;
    int ncols;
    RASTER_MAP_TYPE data_type;
{
    return embed_given_nulls(rast, null_row, data_type, ncols);
}

static int
embed_given_nulls(cell, nulls, map_type, ncols)
    void *cell;
    char *nulls;
    RASTER_MAP_TYPE map_type;
    int ncols;
{
    int i, ind;
    CELL *c;
    FCELL *f;
    DCELL *d;

    c = (CELL *) cell;
    f = (FCELL *) cell;
    d = (DCELL *) cell;

    ind = F2I(map_type);

    for(i = 0; i < ncols; i++)
    {
      if(nulls[i])
         switch (ind)
         {
            case 0:  G_set_c_null_value(c+i, 1); break;
            case 1:  G_set_f_null_value(f+i, 1); break;
            case 2:  G_set_d_null_value(d+i, 1); break;
         }
    }
    return 1;
}

